@@ -31,23 +31,48 @@ import gconf
 import dbus
 
 import os
-import signal
 import sys
+import signal
 from thread import start_new_thread
 from time import sleep
+import posix
 
 import globalhotkeys
 from simplegladeapp import SimpleGladeApp, bindtextdomain
 from prefs import PrefsDialog, LKEY, GKEY
-from common import *
-from guake_globals import *
+from dbusiface import DbusManager, DBUS_NAME, DBUS_PATH
+from common import test_gconf, pixmapfile, gladefile, ShowableError, _
+from guake_globals import NAME, VERSION, LOCALE_DIR, KEY, GCONF_PATH, \
+    TERMINAL_MATCH_EXPRS, TERMINAL_MATCH_TAGS, \
+    ALIGN_LEFT, ALIGN_RIGHT, ALIGN_CENTER
 
 pynotify.init('Guake!')
 
 GNOME_FONT_PATH = '/desktop/gnome/interface/monospace_font_name'
 
 # Loading translation
-bindtextdomain(name, locale_dir)
+bindtextdomain(NAME, LOCALE_DIR)
+
+
+class PromptQuitDialog(gtk.MessageDialog):
+    """Prompts the user whether to quit or not if there are procs running.
+    """
+    def __init__(self, parent, running_procs):
+        super(PromptQuitDialog, self).__init__(
+            parent,
+            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
+            gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO)
+
+        self.set_keep_above(True)
+        self.set_markup(_('Do you really want to quit Guake!?'))
+        if running_procs == 1:
+            self.format_secondary_markup(
+                _("<b>There is one process still running.</b>")
+            )
+        elif running_procs > 1:
+            self.format_secondary_markup(
+                _("<b>There are %d processes running.</b>" % running_procs)
+            )
 
 
 class AboutDialog(SimpleGladeApp):
@@ -64,7 +89,7 @@ class AboutDialog(SimpleGladeApp):
         dialog.set_property('logo', img)
 
         dialog.set_name('Guake!')
-        dialog.set_version(version)
+        dialog.set_version(VERSION)
 
 
 class GConfHandler(object):
@@ -93,7 +118,7 @@ class GConfHandler(object):
         notify_add(KEY('/general/use_trayicon'), self.trayicon_toggled)
         notify_add(KEY('/general/window_ontop'), self.ontop_toggled)
         notify_add(KEY('/general/window_tabbar'), self.tabbar_toggled)
-        notify_add(KEY('/general/window_size'), self.size_changed)
+        notify_add(KEY('/general/window_height'), self.size_changed)
 
         notify_add(KEY('/general/use_scrollbar'), self.scrollbar_toggled)
         notify_add(KEY('/general/history_size'), self.history_size_changed)
@@ -105,7 +130,8 @@ class GConfHandler(object):
         notify_add(KEY('/style/font/color'), self.fcolor_changed)
         notify_add(KEY('/style/background/color'), self.bgcolor_changed)
         notify_add(KEY('/style/background/image'), self.bgimage_changed)
-        notify_add(KEY('/style/background/transparency'), self.bgtransparency_changed)
+        notify_add(KEY('/style/background/transparency'),
+                   self.bgtransparency_changed)
 
         notify_add(KEY('/general/compat_backspace'), self.backspace_changed)
         notify_add(KEY('/general/compat_delete'), self.delete_changed)
@@ -141,11 +167,20 @@ class GConfHandler(object):
         else:
             self.guake.toolbar.hide()
 
-    def size_changed(self, client, connection_id, entry, data):
-        """If the gconf var window_size be changed, this method will
-        be called and will call the resize function in guake.
+    def alignment_changed(self, client, connection_id, entry, data):
+        """If the gconf var window_halignment be changed, this method will
+        be called and will call the move function in guake.
         """
         window_rect = self.guake.get_final_window_rect()
+        self.guake.window.move(window_rect.x, window_rect.y)
+
+    def size_changed(self, client, connection_id, entry, data):
+        """If the gconf var window_height or window_width are changed,
+        this method will be called and will call the resize function
+        in guake.
+        """
+        window_rect = self.guake.get_final_window_rect()
+        self.guake.window.move(window_rect.x, window_rect.y)
         self.guake.window.resize(window_rect.width, window_rect.height)
 
     def scrollbar_toggled(self, client, connection_id, entry, data):
@@ -337,7 +372,7 @@ class GConfKeyHandler(object):
         key, mask = gtk.accelerator_parse(gets('quit'))
         if key > 0:
             self.accel_group.connect_group(key, mask, gtk.ACCEL_VISIBLE,
-                                           gtk.main_quit)
+                                           self.guake.accel_quit)
 
         key, mask = gtk.accelerator_parse(gets('new_tab'))
         if key > 0:
@@ -388,6 +423,7 @@ class GuakeTerminal(vte.Terminal):
         self.configure_terminal()
         self.add_matches()
         self.connect('button-press-event', self.button_press)
+        self.matched_value = ''
 
     def configure_terminal(self):
         """Sets all customized properties on the terminal
@@ -434,9 +470,8 @@ class GuakeTerminal(vte.Terminal):
             elif TERMINAL_MATCH_TAGS[tag] == 'email':
                 value = 'mailto:%s' % value
 
-            # I'm temporarely using this little hammer because
-            # gtk_show_uri seem to not be binded to python yet.
-            open_uri(value)
+            gtk.show_uri(self.window.get_screen(), value,
+                         gtk.gdk.x11_get_server_time(self.window))
         elif event.button == 3 and matched_string:
             self.matched_value = matched_string[0]
 
@@ -477,7 +512,7 @@ class Guake(SimpleGladeApp):
         # trayicon!
         img = pixmapfile('guake-tray.png')
         self.tray_icon = gtk.status_icon_new_from_file(img)
-        self.tray_icon.set_tooltip(_('Guake-Terminal'))
+        self.tray_icon.set_tooltip(_('Guake Terminal'))
         self.tray_icon.connect('popup-menu', self.show_menu)
         self.tray_icon.connect('activate', self.show_hide)
 
@@ -541,6 +576,9 @@ class Guake(SimpleGladeApp):
         # user tries to use the context menu.
         self.showing_context_menu = False
         def hide_context_menu(menu):
+            """Turn context menu flag off to make sure it is not being
+            shown.
+            """
             self.showing_context_menu = False
         self.get_widget('context-menu').connect('hide', hide_context_menu)
         self.get_widget('tab-menu').connect('hide', hide_context_menu)
@@ -581,10 +619,25 @@ class Guake(SimpleGladeApp):
             # unset in the preferences windows)
             notification = pynotify.Notification(
                 _('Guake!'),
-                _('Guake is already running,\n'
+                _('Guake is now running,\n'
                   'press <b>%s</b> to use it.') % label, filename)
             notification.show()
 
+    def execute_command(self, command, tab=None):
+        """Execute the `command' in the `tab'. If tab is None, the
+        command will be executed in the currently selected
+        tab. Command should end with '\n', otherwise it will be
+        appended to the string.
+        """
+        if not self.term_list:
+            self.add_tab()
+
+        if command[-1] != '\n':
+            command += '\n'
+
+        index = self.notebook.get_current_page()
+        self.term_list[tab or index].feed_child(command)
+
     def on_resizer_drag(self, widget, event):
         """Method that handles the resize drag. It does not actuall
         moves the main window. It just set the new window size in
@@ -600,7 +653,7 @@ class Guake(SimpleGladeApp):
         if percent < 1:
             percent = 1
 
-        self.client.set_int(KEY('/general/window_size'), int(percent))
+        self.client.set_int(KEY('/general/window_height'), int(percent))
 
     def on_window_losefocus(self, window, event):
         """Hides terminal main window when it loses the focus and if
@@ -609,14 +662,16 @@ class Guake(SimpleGladeApp):
         value = self.client.get_bool(KEY('/general/window_losefocus'))
         visible = window.get_property('visible')
         if value and visible and not self.showing_context_menu:
-            self.losefocus_time = gtk.gdk.x11_get_server_time(self.window.window)
+            self.losefocus_time = \
+                gtk.gdk.x11_get_server_time(self.window.window)
             self.hide()
 
-    def show_menu(self, *args):
+    def show_menu(self, status_icon, button, activate_time):
         """Show the tray icon menu.
         """
         menu = self.get_widget('tray-menu')
-        menu.popup(None, None, None, 3, gtk.get_current_event_time())
+        menu.popup(None, None, gtk.status_icon_position_menu,
+                   button, activate_time, status_icon)
 
     def show_context_menu(self, terminal, event):
         """Show the context menu, only with a right click on a vte
@@ -636,6 +691,16 @@ class Guake(SimpleGladeApp):
         context_menu.popup(None, None, None, 3, gtk.get_current_event_time())
         return True
 
+    def show_rename_dialog(self, target, event):
+        """On double-click over a tab, show the rename dialog.
+        """
+        if event.button == 1:
+            if event.type == gtk.gdk._2BUTTON_PRESS:
+                self.accel_rename()
+                self.set_terminal_focus()
+                self.selected_tab.pressed()
+                return
+
     def show_tab_menu(self, target, event):
         """Shows the tab menu with a right click. After that, the
         focus come back to the terminal.
@@ -690,9 +755,9 @@ class Guake(SimpleGladeApp):
             self.add_tab()
 
         window_rect = self.get_final_window_rect()
+        self.window.move(window_rect.x, window_rect.y)
         self.window.resize(window_rect.width, window_rect.height)
         self.window.show_all()
-        self.window.move(window_rect.x, window_rect.y)
 
         try:
             # does it work in other gtk backends
@@ -716,30 +781,57 @@ class Guake(SimpleGladeApp):
 
     def get_final_window_rect(self):
         """Gets the final size of the main window of guake. The height
-        is just the window_size property. But width is calculated as
-        100% of the screen and tested against the monitor geometry
-        width.
+        is the window_height property, width is window_width and the
+        horizontal alignment is given by window_alignment.
         """
         screen = self.window.get_screen()
-        height = self.client.get_int(KEY('/general/window_size'))
+        height = self.client.get_int(KEY('/general/window_height'))
+        width = 100
+        halignment = self.client.get_int(KEY('/general/window_halignment'))
 
         # get the rectangle just from the first/default monitor in the
         # future we might create a field to select which monitor you
         # wanna use
         window_rect = screen.get_monitor_geometry(0)
-
+        total_width = window_rect.width
         window_rect.height = window_rect.height * height / 100
+        window_rect.width = window_rect.width * width / 100
+
+        if width < total_width:
+            if halignment == ALIGN_CENTER:
+                window_rect.x = (total_width - window_rect.width) / 2
+            elif halignment == ALIGN_LEFT:
+                window_rect.x = 0
+            elif halignment == ALIGN_RIGHT:
+                window_rect.x = total_width - window_rect.width
+        window_rect.y = 0
         return window_rect
 
+    def get_running_fg_processes(self):
+        """Get the number processes for each terminal/tab. The code is taken
+        from gnome-terminal.
+        """
+        total_procs = 0
+        term_idx = 0
+        for terminal in self.term_list:
+            fdpty = terminal.get_pty()
+            term_pid = self.pid_list[term_idx]
+            fgpid = posix.tcgetpgrp(fdpty)
+            if not (fgpid == -1 or fgpid == term_pid):
+            	total_procs += 1
+            term_idx += 1
+        return total_procs
+
     # -- configuration --
 
     def load_config(self):
         """"Just a proxy for all the configuration stuff.
         """
         self.client.notify(KEY('/general/use_trayicon'))
+        self.client.notify(KEY('/general/prompt_on_quit'))
         self.client.notify(KEY('/general/window_tabbar'))
         self.client.notify(KEY('/general/window_ontop'))
-        self.client.notify(KEY('/general/window_size'))
+        self.client.notify(KEY('/general/window_height'))
         self.client.notify(KEY('/general/use_scrollbar'))
         self.client.notify(KEY('/general/history_size'))
         self.client.notify(KEY('/general/show_resizer'))
@@ -749,6 +841,24 @@ class Guake(SimpleGladeApp):
         self.client.notify(KEY('/style/background/image'))
         self.client.notify(KEY('/style/background/transparency'))
         self.client.notify(KEY('/general/use_default_font'))
+        self.client.notify(KEY('/general/compat_backspace'))
+        self.client.notify(KEY('/general/compat_delete'))
+
+    def accel_quit(self, *args):
+        """Callback to prompt the user whether to quit Guake or not.
+        """
+        if self.client.get_bool(KEY('/general/prompt_on_quit')):
+            procs = self.get_running_fg_processes()
+            if procs >= 1:
+                dialog = PromptQuitDialog(self.window, procs)
+                response = dialog.run() == gtk.RESPONSE_YES
+                dialog.destroy()
+                if response:
+                    gtk.main_quit()
+            else:
+                gtk.main_quit()
+        else:
+            gtk.main_quit()
 
     def accel_add(self, *args):
         """Callback to add a new tab. Called by the accel key.
@@ -837,6 +947,10 @@ class Guake(SimpleGladeApp):
     # -- callbacks --
 
     def on_terminal_exited(self, term, widget):
+        """When a terminal is closed, shell process should be killed,
+        this is the method that does that, or, at least calls
+        `delete_tab' method to do the work.
+        """
         self.delete_tab(self.notebook.page_num(widget), kill=False)
 
     def on_rename_activate(self, *args):
@@ -888,6 +1002,13 @@ class Guake(SimpleGladeApp):
         pagepos = self.notebook.get_current_page()
         self.delete_tab(pagepos)
 
+    def rename_current_tab(self, new_text):
+        """Sets the `self.selected_tab' var with the selected radio
+        button and change its label to `new_text'.
+        """
+        pagepos = self.notebook.get_current_page()
+        self.selected_tab = self.tabs.get_children()[pagepos]
+        self.selected_tab.set_label(new_text)
 
     def get_current_dir(self):
         """Gets the working directory of the current tab to create a
@@ -896,18 +1017,20 @@ class Guake(SimpleGladeApp):
         active_pagepos = self.notebook.get_current_page()
         directory = os.path.expanduser('~')
         if active_pagepos >= 0:
-            cwd = "/proc/%d/cwd" % self.pid_list[active_pagepos]
+            cwd = os.readlink("/proc/%d/cwd" % self.pid_list[active_pagepos])
             if os.path.exists(cwd):
                 directory = cwd
         return directory
 
-    def get_fork_params(self):
+    def get_fork_params(self, default_params=None):
         """Return all parameters to be passed to the fork_command
-        method of a vte terminal.
+        method of a vte terminal. Params returned can be expanded by
+        the `params' parameter that receive a dictionary.
         """
         # use dictionary to pass named params to work around command
         # parameter in fork_command not accepting None as argument.
-        # When we pass None as command, vte starts the default user shell.
+        # When we pass None as command, vte starts the default user
+        # shell.
         params = {}
 
         shell = self.client.get_string(KEY('/general/default_shell'))
@@ -918,12 +1041,17 @@ class Guake(SimpleGladeApp):
         if login_shell:
             params['argv'] = ['-']
 
-        params['directory'] = self.get_current_dir()
+        if self.client.get_bool(KEY('/general/open_tab_cwd')):
+            params['directory'] = self.get_current_dir()
         params['loglastlog'] = login_shell
 
+        # Leting caller change/add values to fork params.
+        if default_params:
+            params.update(default_params)
+
         return params
 
-    def add_tab(self, *args):
+    def add_tab(self, directory=None):
         """Adds a new tab to the terminal notebook.
         """
         box = GuakeTerminalBox()
@@ -932,10 +1060,18 @@ class Guake(SimpleGladeApp):
         box.terminal.connect('child-exited', self.on_terminal_exited, box)
         box.show()
 
-        last_added = len(self.term_list)
         self.term_list.append(box.terminal)
 
-        pid = box.terminal.fork_command(**self.get_fork_params())
+        # We can choose the directory to vte launch. It is important
+        # to be used by dbus interface. I'm testing if directory is a
+        # string because when binded to a signal, the first param can
+        # be a button not a directory.
+        default_params = {}
+        if isinstance(directory, basestring):
+            default_params['directory'] = directory
+
+        final_params = self.get_fork_params(default_params)
+        pid = box.terminal.fork_command(**final_params)
         self.pid_list.append(pid)
 
         # Adding a new radio button to the tabbar
@@ -947,6 +1083,7 @@ class Guake(SimpleGladeApp):
         bnt.set_property('can-focus', False)
         bnt.set_property('draw-indicator', False)
         bnt.connect('button-press-event', self.show_tab_menu)
+        bnt.connect('button-press-event', self.show_rename_dialog)
         bnt.connect('clicked',
                     lambda *x: self.notebook.set_current_page(
                         self.notebook.page_num(box)
@@ -989,7 +1126,13 @@ class Guake(SimpleGladeApp):
         num_tries = 30
 
         while num_tries > 0:
-            if os.waitpid(pid, os.WNOHANG)[0] != 0:
+            try:
+                # Try to wait for the pid to be closed. If it does not
+                # exist anymore, an OSError is raised and we can
+                # safely ignore it.
+                if os.waitpid(pid, os.WNOHANG)[0] != 0:
+                    break
+            except OSError:
                 break
             sleep(0.1)
             num_tries -= 1
@@ -1016,13 +1159,31 @@ class Guake(SimpleGladeApp):
         """
         self.tabs.get_children()[page].set_active(True)
 
+    def select_tab(self, tab_index):
+        """Select an already added tab by its index.
+        """
+        try:
+            self.tabs.get_children()[tab_index].set_active(True)
+        except IndexError:
+            pass
+
+    def get_selected_tab(self):
+        """return the selected tab index, it also set the
+        self.selected_tab var.
+        """
+        pagepos = self.notebook.get_current_page()
+        self.selected_tab = self.tabs.get_children()[pagepos]
+        return pagepos
+
 def main():
     """Parses the command line parameters and decide if dbus methods
-    should be called or not.
+    should be called or not. If there is already a guake instance
+    running it will be used and a True value will be returned,
+    otherwise, false will be returned.
     """
     from optparse import OptionParser
     parser = OptionParser()
-    parser.add_option('-s', '--show-hide', dest='show_hide',
+    parser.add_option('-t', '--toggle-visibility', dest='show_hide',
             action='store_true', default=False,
             help=_('Toggles the visibility of the terminal window'))
 
@@ -1034,49 +1195,96 @@ def main():
             action='store_true', default=False,
             help=_('Shows Guake\'s about info'))
 
+    parser.add_option('-n', '--new-tab', dest='new_tab',
+            action='store', default='',
+            help=_('Add a new tab'))
+
+    parser.add_option('-s', '--select-tab', dest='select_tab',
+            action='store', default='',
+            help=_('Select a tab'))
+
+    parser.add_option('-g', '--selected-tab', dest='selected_tab',
+            action='store_true', default=False,
+            help=_('Return the selectd tab index.'))
+
+    parser.add_option('-e', '--execute-command', dest='command',
+            action='store', default='',
+            help=_('Execute an arbitrary command in the selected tab.'))
+
+    parser.add_option('-r', '--rename-tab', dest='rename_tab',
+            action='store', default='',
+            help=_('Rename the selected tab.'))
+
     parser.add_option('-q', '--quit', dest='quit',
             action='store_true', default=False,
             help=_('Says to Guake go away =('))
 
-    options, args = parser.parse_args()
+    options = parser.parse_args()[0]
 
-    bus = dbus.SessionBus()
+    # Trying to get an already running instance of guake. If it is not
+    # possible, lets create a new instance. This function will return
+    # a boolean value depending on this decision.
     try:
-        remote_object = bus.get_object('org.gnome.Guake.DBus', '/DBusInterface')
+        bus = dbus.SessionBus()
+        remote_object = bus.get_object(DBUS_NAME, DBUS_PATH)
+        already_running = True
     except dbus.DBusException:
-        return
+        instance = Guake()
+        remote_object = DbusManager(instance)
+        already_running = False
+
+    called_with_param = False
 
     if options.show_hide:
         remote_object.show_hide()
-        sys.exit(0)
+        called_with_param = True
 
     if options.show_preferences:
         remote_object.show_prefs()
-        sys.exit(0)
+        called_with_param = True
+
+    if options.new_tab:
+        remote_object.add_tab(options.new_tab)
+        called_with_param = True
+
+    if options.select_tab:
+        selected = int(options.select_tab)
+        remote_object.select_tab(selected)
+        called_with_param = True
+
+    if options.selected_tab:
+        selected = remote_object.get_selected_tab()
+        sys.stdout.write('%d\n' % selected)
+        called_with_param = True
+
+    if options.command:
+        remote_object.execute_command(options.command)
+        called_with_param = True
+
+    if options.rename_tab:
+        remote_object.rename_current_tab(options.rename_tab)
+        called_with_param = True
 
     if options.show_about:
         remote_object.show_about()
-        sys.exit(0)
+        called_with_param = True
 
     if options.quit:
         remote_object.quit()
-        sys.exit(0)
+        called_with_param = True
 
-    # here we know that guake was called without any parameter and it is
-    # already running, so, lets toggle glade visibility and exit!
-    remote_object.show_hide()
-    sys.exit(0)
+    if not called_with_param and already_running:
+        # here we know that guake was called without any parameter and
+        # it is already running, so, lets toggle its visibility.
+        remote_object.show_hide()
 
+    return already_running
 
 if __name__ == '__main__':
-    from dbusiface import dbus_init
-    main()
-
     if not test_gconf():
         raise ShowableError(_('Guake can not init!'),
             _('Gconf Error.\n'
               'Have you installed <b>guake.schemas</b> properlly?'))
 
-    guake = Guake()
-    dbus_init(guake)
-    guake.run()
+    if not main():
+        gtk.main()
