@@ -25,7 +25,7 @@ import gobject
 import gconf
 
 from simplegladeapp import SimpleGladeApp
-from guake_globals import GCONF_PATH, KEY
+from guake_globals import GCONF_PATH, KEY, ALIGN_LEFT, ALIGN_RIGHT, ALIGN_CENTER
 from common import *
 import globalhotkeys
 
@@ -113,6 +113,12 @@ class PrefsCallbacks(object):
         """
         self.client.set_bool(KEY('/general/use_login_shell'), chk.get_active())
 
+    def on_open_tab_cwd_toggled(self, chk):
+        """Changes the activity of open_tab_cwd in gconf
+        """
+        self.client.set_bool(KEY('/general/open_tab_cwd'), chk.get_active())
+
+
     def on_use_trayicon_toggled(self, chk):
         """Changes the activity of use_trayicon in gconf
         """
@@ -138,11 +144,16 @@ class PrefsCallbacks(object):
         """
         self.client.set_bool(KEY('/general/window_tabbar'), chk.get_active())
 
-    def on_window_size_value_changed(self, hscale):
-        """Changes the value of window_size in gconf
+    def on_window_height_value_changed(self, hscale):
+        """Changes the value of window_height in gconf
         """
         val = hscale.get_value()
-        self.client.set_int(KEY('/general/window_size'), int(val))
+        self.client.set_int(KEY('/general/window_height'), int(val))
+
+    def on_prompt_on_quit_toggled(self, chk):
+        """Set the `prompt on quit' property in gconf
+        """
+        self.client.set_bool(KEY('/general/prompt_on_quit'), chk.get_active())
 
     # scrolling tab
 
@@ -385,6 +396,10 @@ class PrefsDialog(SimpleGladeApp):
         value = self.client.get_bool(KEY('/general/use_popup'))
         self.get_widget('use_popup').set_active(value)
 
+        # prompt on quit
+        value = self.client.get_bool(KEY('/general/prompt_on_quit'))
+        self.get_widget('prompt_on_quit').set_active(value)
+
         # ontop
         value = self.client.get_bool(KEY('/general/window_ontop'))
         self.get_widget('window_ontop').set_active(value)
@@ -397,10 +412,6 @@ class PrefsDialog(SimpleGladeApp):
         value = self.client.get_bool(KEY('/general/window_tabbar'))
         self.get_widget('window_tabbar').set_active(value)
 
-        # size
-        value = float(self.client.get_int(KEY('/general/window_size')))
-        self.get_widget('window_size').set_value(value)
-
         # scrollbar
         value = self.client.get_bool(KEY('/general/use_scrollbar'))
         self.get_widget('use_scrollbar').set_active(value)
@@ -618,6 +629,22 @@ class KeyEntry(object):
         return self.keycode == rval.keycode and \
             self.mask == rval.mask
 
+def setup_standalone_signals(instance):
+    """Called when prefs dialog is running in standalone mode. It
+    makes the delete event of dialog and click on close button finish
+    the application.
+    """
+    window = instance.get_widget('config-window')
+    window.connect('delete-event', gtk.main_quit)
+
+    # We need to block the execution of the already associated
+    # callback before connecting the new handler.
+    button = instance.get_widget('button1')
+    button.handler_block_by_func(instance.gtk_widget_destroy)
+    button.connect('clicked', gtk.main_quit)
+
+    return instance
+
 if __name__ == '__main__':
-    PrefsDialog().show()
+    setup_standalone_signals(PrefsDialog()).show()
     gtk.main()
